
from django.urls import path, include

from exchange import views


app_name = 'exchange'


urlpatterns = [

    path('set-currency/', views.set_currency, name='set-currency')

]


app_urls = [
    path('exchange/', include((urlpatterns, app_name)))
]





import re

from exchange.constants import (
    CURRENCIES,
    CURRENCY_NAMES,
    DEFAULT_CURRENCY,
    CURRENCY_EUR,
    CURRENCY_UAH,
    CURRENCY_USD,
    CURRENCY_SESSION_KEY)


def round_number(value, decimal_places=2, down=False):

    assert decimal_places > 0
    factor = 1.0 ** decimal_places
    sign = -1 if value < 0 else 1
    return int(value * factor + sign * (0 if down else 0.5)) / factor


def format_number(value):

    append_comma = lambda match_object: "%s," % match_object.group(0)

    value = "%.2f" % float(value)
    value = re.sub("(\d)(?=(\d{3})+\.)", append_comma, value)

    return value


def format_price(price, round_price=False):
    price = float(price)
    return format_number(round_number(price) if round_price else price)


def format_printable_price(price, currency=DEFAULT_CURRENCY):
    return '%s %s' % (format_price(price), dict(CURRENCIES)[currency])


def get_currency_from_session(session):
    currency = session.get(CURRENCY_SESSION_KEY) or DEFAULT_CURRENCY
    return int(currency)


def get_price_factory(rates, src, dst):

    if src == dst:
        return lambda p: p

    name = lambda c: CURRENCY_NAMES[c]

    if src == CURRENCY_UAH:
        return lambda p: p / getattr(rates, name(dst))

    if dst == CURRENCY_UAH:
        return lambda p: p * getattr(rates, name(src))

    if src == CURRENCY_USD and dst == CURRENCY_EUR:
        return lambda p: p * rates.usd_eur

    if src == CURRENCY_EUR and dst == CURRENCY_USD:
        return lambda p: p / rates.usd_eur

    raise ValueError('Unknown currencies')




from exchange.utils import get_currency_from_session
from exchange.constants import CURRENCIES


def currencies(request):
    return {
        'CURRENCY': get_currency_from_session(request.session),
        'CURRENCIES': CURRENCIES
    }




from django.apps import AppConfig
from django.utils.translation import ugettext_lazy as _


def setup_settings(settings, is_prod, **kwargs):

    if 'solo' not in settings['INSTALLED_APPS']:
        settings['INSTALLED_APPS'] += ['solo']

    settings['CONTEXT_PROCESSORS'] += [
        'exchange.context_processors.currencies'
    ]


class ExchangeAppConfig(AppConfig):

    name = 'exchange'
    verbose_name = _('Exchange')


default_app_config = 'exchange.ExchangeAppConfig'




from django import template

from exchange.utils import get_currency_from_session
from exchange.forms import CurrencyForm


register = template.Library()


@register.simple_tag(takes_context=True)
def get_currency_form(context):
    currency = get_currency_from_session(context.request.session)
    return CurrencyForm(initial={'currency': currency})
